# Copyright (c) 2013 Galah Group LLC
# Copyright (c) 2013 Other contributers as noted in the CONTRIBUTERS file
#
# This file is part of galah-interact-python.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

class Block:
    """Represents a block of code."""

    def __init__(self, lines, sub_blocks = None):
        if sub_blocks is None:
            sub_blocks = []

        self.lines = lines
        self.sub_blocks = sub_blocks

    def _to_str_list(self, indent_level = 0):
        result = [("\t" * indent_level) + repr(i) for i in self.lines]
        for i in self.sub_blocks:
            result += i._to_str_list(indent_level = indent_level + 1)

        return result

    def __str__(self):
        return "\n".join(self._to_str_list())

class Line:
    """Represents a line of code."""

    def __init__(self, line_number, code):
        self.code = code
        self.line_number = line_number

    def __str__(self):
        return self.code

    def __repr__(self):
        return "Line(%d, %s)" % (self.line_number, repr(self.code))

    def indent_level(self):
        """
        Determine the indentation level of the current line. Does this by
        counting the number of tabs and spaces at the start of the line and
        adding the two numbers together. If the line is blank (not including
        white space), None is returned.

        """

        # Iterates through the line character by character until a
        # non-whitespace character is hit.
        for i, c in enumerate(self.code):
            if c not in [" ", "\t"]:
                break
        else:
            # If we never hit a non-whitespace character...
            return None

        # i is the position of the first non-whitespace character, and because i
        # is zero-indexed, i is also the number of whitespace characters we saw.
        return i

    @classmethod
    def make_lines(cls, lines, start = 1):
        return (cls(n, line) for n, line in enumerate(lines))

    @staticmethod
    def lines_to_str_list(lines):
        return (i.code for i in lines)

    @staticmethod
    def lines_to_str(lines):
        return "\n".join(Line.lines_to_str_list(lines))

    def __eq__(self, other):
        return self.code == other.code and self.line_number == other.line_number

def grab_blocks(lines):
    """
    Finds all blocks created using curly braces (does not handle two line if
    statements for example). Returns a single Block object which can be treated
    as a tree.

    """

    # The number of nested blocks the current line is in relative to our
    # starting point. We will only look at the indentation for lines with
    # in_block == 0, all other lines we will recursively defer.
    in_block = 0

    # These are all of the lines that are within the current block (and not
    # within any sub blocks).
    lines_to_check = []

    # Contains the actual result (the list of two-tuples).
    sub_blocks = []

    # When we encounter code that is in a sub block, we push it onto this list.
    # Then when we get out of that sub block, we recurse on these lines and then
    # empty the list.
    unhandled_chunk = []

    for line in lines:
        # Remove all quoted strings from the line, that way when we search for
        # a curly brace we know it's an actual curly brace.
        stripped_line = cleanse_quoted_strings(line.code)

        # Will be set to true if the the current line is in the current block.
        include_current_line = False

        # Figure out if we are ending a block here
        for char in stripped_line:
            # If at any point we reach 0, that means we want to include this
            # line in the current block. This may occur if you have something
            # like "} else {".
            if in_block == 0:
                include_current_line = True

            # See comment above the in_block initialization.
            if char == "{":
                in_block += 1
            elif char == "}":
                in_block -= 1

        # Check if we ended a block here.
        if in_block == 0:
            include_current_line = True

        if include_current_line:
            # If we were just looking at a chunk of code in a sub block,
            # recurse properly.
            if unhandled_chunk:
                unhandled_block = grab_blocks(unhandled_chunk)
                if unhandled_block:
                    sub_blocks.append(unhandled_block)

                unhandled_chunk = []

            lines_to_check.append(line)
        else:
            unhandled_chunk.append(line)

    if not lines_to_check and not sub_blocks:
        return None
    else:
        return Block(lines_to_check, sub_blocks)

def cleanse_quoted_strings(line):
    """
    Removes all quoted strings from the line. Single quotes are treated the
    same as double quotes.

    Escaped quotes are handled. A forward slash is assumed to be the escape
    character. Escape sequences are not processed (meaning `\"` does not become
    `"`, it just remains as `\"`).

    """

    # Returns ' if " is given, returns " if ' is given.
    inv_quote = lambda x: "'" if x == "\"" else "\""

    is_quote = lambda x: x in ["\"", "'"]

    # Will be a list of characters that we will join together to get the
    # resulting string sans quoted strings.
    unquoted_string = []

    in_quotes = None
    for i, char in enumerate(line):
        # Check to see if this character is escaped (this will occur if there is
        # an odd number of back slashes in front of it).
        num_slashes = 0
        for j in reversed(range(0, i)):
            if line[j] == "\\":
                num_slashes += 1
            else:
                break
        escaped = num_slashes % 2 == 1

        if char == in_quotes and not escaped:
            in_quotes = None
            continue
        elif is_quote(char) and in_quotes is None:
            in_quotes = char
            continue

        if in_quotes is None:
            unquoted_string.append(char)

    return "".join(unquoted_string)



# Copyright (c) 2013 Galah Group LLC
# Copyright (c) 2013 Other contributers as noted in the CONTRIBUTERS file
#
# This file is part of galah-interact-python.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from interact._utils import resolve_path
from interact.swig import *

import os
import imp
import inspect

import atexit
import shutil
import tempfile

to_delete = []

def _cleanup():
    for i in to_delete:
        shutil.rmtree(i)
atexit.register(_cleanup)

def load_cpp_files(cpp_files):
    module_dict = {}
    modules = []

    temp_dir = tempfile.mkdtemp()
    os.chdir(temp_dir)

    for f in cpp_files:
        modules.append(generate_swig_interface(f))

    interface_files = ("%s.i" % (module) for module in modules)
    generate_swig_wrappers(interface_files)
    generate_shared_libraries(modules)

    for module in modules:
        module_dict[module] = {}
        mod = imp.load_source(module, '%s.py' %
                              (temp_dir + '/' + module))

        # Get all functions and classes in this module
        for name, impl in inspect.getmembers(mod,
                                             lambda a: (inspect.isbuiltin(a) or
                                                        inspect.isclass(a))):
            module_dict[module][name] = impl

    to_delete.append(temp_dir)

    return module_dict



# Copyright (c) 2013 Galah Group LLC
# Copyright (c) 2013 Other contributers as noted in the CONTRIBUTERS file
#
# This file is part of galah-interact-python.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import operator

def test_cases_one_of(
        testcase_object, test_cases, test_func, comparator = operator.eq):
    for case, expected in test_cases:
        result = test_func(case)
        for i in expected:
            if comparator(result, i):
                break
        else:
            test_case_object.fail(
                "Expected one of %s, got %s for case %s." %
                    (repr(expected), repr(result), repr(case))
            )

def test_cases(testcase_object, test_cases, test_func, assertion = None):
    if assertion is None:
        assertion = testcase_object.assertEqual

    for case, expected in test_cases:
        assertion(test_func(case), expected)



# Copyright (c) 2013 Galah Group LLC
# Copyright (c) 2013 Other contributers as noted in the CONTRIBUTERS file
#
# This file is part of galah-interact-python.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Functions and classes that are essential when using this library. Is imported
by core such that `interact.core.x` and `interact.x` is equivalent.

"""

import _utils
import os.path
import sys

class TestResult:
    """
    Represents the result of one unit of testing. The goal is to generate a
    number of these and then pass them all out of the test harness with a final
    score.

    """

    class Message:
        """
        A message to the user. This is the primary mode of giving feedback to
        users.

        """

        def __init__(self, text, *args, **kwargs):
            # We have some special keyword arguments that we want to snatch if
            # present.
            self.dscore = kwargs.get("dscore", None)
            self.type = kwargs.get("type", None)

            self.args = args
            self.kwargs = kwargs

            self.text = text

        def __str__(self):
            return self.text.format(*self.args, **self.kwargs)

        def __repr__(self):
            return _utils.default_repr(self)

    def __init__(
            self, brief = None, score = None, max_score = None, messages = None,
            default_message = None, bulleted_messages = True):
        if messages is None:
            messages = []

        self.brief = brief
        self.score = score
        self.max_score = max_score
        self.messages = messages
        self.default_message = default_message
        self.bulleted_messages = bulleted_messages

    def add_message(self, *args, **kwargs):
        """
        Adds a message object to the TestResult. If a Message object that is
        used, otherwise a new Message object is constructed and its constructor
        is passed all the arguments.

        """

        if len(args) == 1 and isinstance(args[0], TestResult.Message):
            self.messages.append(args[0])
        else:
            self.messages.append(TestResult.Message(*args, **kwargs))

    def calculate_score(self, starting_score = None, max_score = None):
        """
        Automatically calculates the score by adding up the dscore fields of
        every message.

        If max_score is None, self.max_score is used.

        If starting_score is None, max_score is used.

        """
        if max_score is None:
            max_score = self.max_score

        if starting_score is None:
            starting_score = max_score

        self.score = starting_score
        self.max_score = max_score

        for i in self.messages:
            if i.dscore is not None:
                self.score += i.dscore

    def set_passing(self, passing):
        self.score = 1 if passing else 0
        self.max_score = 1

    def is_passing(self):
        return self.score != 0

    def is_failing(self):
        return self.score == 0

    def to_galah_dict(self, name):
        return {
            "name": name,
            "score": 0 if self.score is None else self.score,
            "max_score": 0 if self.max_score is None else self.max_score,
            "message": self.to_str(show_score = False)
        }

    def to_str(self, show_score = True):
        result = []

        quick_status = ""
        if show_score and self.score is not None:
            quick_status += "Score: %d" % self.score

            if self.max_score is not None and self.max_score != 0:
                quick_status += " out of %d" % self.max_score

            result += [quick_status, ""]

        if self.brief:
            result += [str(self.brief), ""]

        if self.messages:
            for i in self.messages:
                if self.bulleted_messages:
                    result.append(" * " + str(i))
                else:
                    result.append(str(i))
            result.append("")
        elif self.default_message:
            result += [self.default_message, ""]

        return "\n".join(result[:-1])

    def __str__(self):
        return self.to_str()

    def __repr__(self):
        return _utils.default_repr(self)

class FailedDependencies(TestResult):
    def __init__(self, max_score = 10):
        TestResult.__init__(
            self,
            brief = "This test will only be run if all of the other tests it "
                    "depends on pass first. Fix those tests *before* worrying "
                    "about this one.",
            score = 0,
            max_score = 10
        )

class Harness:
    class Test:
        def __init__(self, name, depends, func, result = None):
            self.name = name
            self.depends = depends
            self.func = func
            self.result = result

    def __init__(self):
        self.sheep_data = {}
        self.tests = {}

    def start(self):
        """
        Marks the start of the test harness. When no command line arguments are
        present, this command will block and read JSON in from stdin. If
        command line arguments are present information will be gathered from
        them.

        """

        if len(sys.argv) == 4 and sys.argv[1] == "--test":
            self.sheep_data["harness_directory"] = \
                _utils.resolve_path(sys.argv[2])

            self.sheep_data["testables_directory"] = \
                _utils.resolve_path(sys.argv[3])
        else:
            import json
            self.sheep_data = json.load(sys.stdin)

    def finish(self, score = None, max_score = None):
        """
        Marks the end of the test harness. When start was not initialized via
        command line arguments, this command will print out the test results in
        a human readable fashion. Otherwise it will print out JSON appropriate
        for Galah to read.

        """

        if score is None or max_score is None:
            new_score = 0
            new_max_score = 0
            for i in self.tests.values():
                if i is not None:
                    if i.result.score:
                        new_score += i.result.score

                    if i.result.max_score:
                        new_max_score += i.result.max_score

            if score is None:
                score = new_score

            if max_score is None:
                max_score = new_max_score

        if len(sys.argv) == 4 and sys.argv[1] == "--test":
            for i in self.tests.values():
                if i.result:
                    print i.result
                    print "-------"
            print "Final result: %d out of %d" % (score, max_score)
        else:
            import json
            results = {
                "score": score,
                "max_score": max_score,
                "tests": []
            }

            for i in self.tests.values():
                if i is not None:
                    results["tests"].append(i.result.to_galah_dict(i.name))

            json.dump(results, sys.stdout)

    def test(self, name, depends = None):
        """
        A decorator that takes in a test name and some dependencies and makes
        the harness aware of it all.

        """

        def test_decorator(func):
            def inner(*args, **kwargs):
                return func(*args, **kwargs)

            self.tests[inner] = Harness.Test(name, depends, inner)

            return inner

        return test_decorator

    def run_tests(self):
        """
        Runs all of the tests the user has registered.

        If there is a cyclic dependency a RuntimeError will be raised.

        """

        remaining_tests = list(self.tests.keys())

        # Naively execute each test in proper order based on their
        # dependencies.
        while remaining_tests:
            # Figure out the next test to run
            to_test = None
            for i in remaining_tests:
                current_test = self.tests[i]

                # If the current test doesn't have any dependencies, let's run
                # it now.
                if not current_test.depends:
                    to_test = current_test
                    break
                else:
                    # Check all the dependencies of the current test, if they
                    # are all passing, we're done.
                    for j in current_test.depends:
                        # If one of the dependencies hasn't been run or it's
                        # failing
                        if self.tests[j].result is None:
                            break
                        elif self.tests[j].result.is_failing():
                            if not isinstance(current_test.result,
                                    FailedDependencies):
                                current_test.result = FailedDependencies()

                            current_test.result.add_message(
                                "Dependency *{dependency_name}* failed.",
                                dependency_name = self.tests[j].name
                            )
                    else:
                        if isinstance(current_test.result, FailedDependencies):
                            remaining_tests.remove(current_test.func)
                        else:
                            to_test = current_test

            if to_test is not None:
                self.tests[to_test.func].result = to_test.func()
                remaining_tests.remove(to_test.func)
            elif remaining_tests:
                raise RuntimeError("Cyclic dependencies!")
            else:
                break

    def student_file(self, filename):
        """
        Given a path to a student's file relative to the root of the student's
        submission, returns an absolute path to that file.

        """

        testables_directory = \
            self.sheep_data.get("testables_directory", "../submission/")

        path = os.path.join(testables_directory, filename)

        # Ensure that we return an absolute path.
        return _utils.resolve_path(path)

    def student_files(self, *args):
        """
        Very similar to student_file. Given many files as arguments, will return
        a list of absolute paths to those files.

        """

        return [self.student_file(i) for i in args]



# Copyright (c) 2013 Galah Group LLC
# Copyright (c) 2013 Other contributers as noted in the CONTRIBUTERS file
#
# This file is part of galah-interact-python.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import interact.parse as parse
import testcore

import unittest
class TestCleanseQuotedStrings(unittest.TestCase):
    def test_basic(self):
        test_cases = (
            (r"""this "is" some "quoted" 'strings"all"'""", "this  some  "),
            ("", ""),
            ("""'""""""hi'hi""", "hi"),
            ("''''''''''''''''", ""),
            ('""""""""""""""""', ""),
            ("I am the 'destroyer \" of muffins'", "I am the "),
            ('Duck "duck\\\" duck" go', "Duck  go")
        )

        testcore.test_cases(
        	self, test_cases, parse.cleanse_quoted_strings
        )

class TestGrabBlocks(unittest.TestCase):
    def test_basic(self):
        from interact.parse import Line, Block
        test_cases = (
            # Simple test case
            (
                [
                    Line(0, "#include <iostream>"),
                    Line(1, ""),
                    Line(2, "using namespace std;"),
                    Line(3, ""),
                    Line(4, "int main() {"),
                    Line(5, '    cout << "Hello world" << endl;'),
                    Line(6, "    return 0"),
                    Line(7, "}")
                ],
                Block(
                    lines = [
                        Line(0, "#include <iostream>"),
                        Line(1, ""),
                        Line(2, "using namespace std;"),
                        Line(3, ""),
                        Line(4, "int main() {"),
                        Line(7, "}")
                    ],
                    sub_blocks = [
                        Block(
                            lines = [
                                Line(5, '    cout << "Hello world" << endl;'),
                                Line(6, "    return 0")
                            ]
                        )
                    ]
                )
            ),

            # Slightly tricky test case
            (
                [
                    Line(0,  "#include <iostream>"),
                    Line(1,  ""),
                    Line(2,  "using namespace std;"),
                    Line(3,  ""),
                    Line(4,  "int main() {"),
                    Line(5,  '    cout << "{" << endl;'),
                    Line(6,  "    if (true)"),
                    Line(7,  "    {"),
                    Line(8,  "        return false;"),
                    Line(9,  "    } else {"),
                    Line(10, "        return true;"),
                    Line(11, "oh noz"),
                    Line(12, "    }"),
                    Line(13, "        pinata"),
                    Line(14, "    return 0"),
                    Line(15, "}")
                ],
                Block(
                    lines = [
                        Line(0,  "#include <iostream>"),
                        Line(1,  ""),
                        Line(2,  "using namespace std;"),
                        Line(3,  ""),
                        Line(4,  "int main() {"),
                        Line(15, "}")
                    ],
                    sub_blocks = [
                        Block(
                            lines = [
                                Line(5,  '    cout << "{" << endl;'),
                                Line(6,  "    if (true)"),
                                Line(7,  "    {"),
                                Line(9,  "    } else {"),
                                Line(12, "    }"),
                                Line(13, "        pinata"),
                                Line(14, "    return 0")
                            ],
                            sub_blocks = [
                                Block(
                                    lines = [
                                        Line(8, "        return false;")
                                    ]
                                ),
                                Block(
                                    lines = [
                                        Line(10, "        return true;"),
                                        Line(11, "oh noz")
                                    ]
                                )
                            ]
                        )
                    ]
                )
            )
        )

        def compare_blocks(a, b):
            if a is None or b is None:
                return a is b

            if a.lines != b.lines:
                return False

            if len(a.sub_blocks) != len(b.sub_blocks):
                return False

            matches = []
            for i in a.sub_blocks:
                for j in b.sub_blocks:
                    if i.lines == j.lines:
                        matches.append((i, j))

            if len(matches) != len(a.sub_blocks):
                return False

            for i in matches:
                if not compare_blocks(*i):
                    return False

            return True

        for case, expected in test_cases:
            if not compare_blocks(parse.grab_blocks(case), expected):
                # TODO: Make the failure output meaningful.
                self.fail("abaabahaha")



# Copyright (c) 2013 Galah Group LLC
# Copyright (c) 2013 Other contributers as noted in the CONTRIBUTERS file
#
# This file is part of galah-interact-python.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

def default_repr(obj):
    """
    Returns a string that would be appropriate to return from a repr() call. The
    string will be of the form `ClassName(attribute1 = value, ...)`.

    """

    attributes = sorted(obj.__dict__.items(), key = lambda x: x[0])
    attribute_strings = []
    for key, value in attributes:
        if not key.startswith("_"):
            attribute_strings.append(key + " = " + repr(value))

    return "%s(%s)" % (obj.__class__.__name__, ", ".join(attribute_strings))

import os.path
def resolve_path(path):
    return os.path.abspath(os.path.expanduser(path))




# Copyright (c) 2013 Galah Group LLC
# Copyright (c) 2013 Other contributers as noted in the CONTRIBUTERS file
#
# This file is part of galah-interact-python.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Test suites that perform full testing on input, returning TestResult objects.
These are typical tests that many harnesses need to perform. Such as checking
indentation or checking to see if the correct files were submitted.

"""

import os
import os.path
import re
import tempfile
import subprocess
import atexit
import shutil

import _utils
import core
import pretty
import parse
import execute

def check_files_exist(*files):
    """
    Checks to see if the given files provided as arguments exist. They must be
    files as defined by os.path.isfile(). Returns a TestResult.

    The TestResult will be passing if and only if all of the files exist.

    """

    missing_files = [i for i in files if not os.path.isfile(i)]

    result = core.TestResult(
        brief = "This test ensures that all of the necessary files are "
                "present.",
        default_message = "Great job! All the necessary files are present."
    )

    if missing_files:
        result.add_message(core.TestResult.Message(
            "You are missing {missing_files_}.",
            missing_files_ = pretty.pretty_list(missing_files),
            missing_files = missing_files,
            dscore = -1,
            type = "interact/filesexist/basic_files_exist"
        ))
        result.set_passing(False)
    else:
        result.set_passing(True)

    return result

def indentation(code, file_name, max_score = 10, allow_negative = False):
    """
    Given a chunk of code as a string, this function will determine if the
    student indented properly. Returns a TestResult object.

    Tests:
        * A student's code is correctly indented if every line of each sub-block
          is indented strictly more than the least indented line on the parent
          block.

          If any problems are found, a single TestMessage will be added to the
          returned TestResult object with type
          `interact/indentation/basic_indent_level`. The lines property of this
          message is a list of every Line that had a problem.

    Test Result:
        The test result object that is returned

    """

    # Transform the code into a list of Line objects
    lines = parse.Line.make_lines(code.splitlines())

    # Break the code up into blocks.
    blocks = parse.grab_blocks(lines)

    # Recursive helper function.
    def check(block, minimum = None):
        problems = []

        # Check that each line in the current block has an indnetation level
        # strictly greater than the minimum.
        for i in block.lines:
            indent_level = i.indent_level()
            if minimum is not None and indent_level is not None and \
                    indent_level <= minimum:
                problems.append(i)

        # Find the indent level of the least indented line in the current block
        levels = []
        for i in block.lines:
            level = i.indent_level()
            if level is not None:
                levels.append(level)
        new_minimum = min(levels)

        # Recurse into every sub block
        for i in block.sub_blocks:
            problems += check(i, new_minimum)

        return problems

    result = core.TestResult(
        brief = "This test checks to ensure you are indenting properly. Make "
                "sure that every time you start a new block (curly braces "
                "delimit blocks) you indent more.",
        default_message = "Great job! You have perfect indentation!",
        max_score = max_score
    )

    problems = check(blocks)
    if problems:
        result.add_message(core.TestResult.Message(
            "{lines_} {line_numbers_} in {file_name} {are_} not indented more "
                "than the outer block.",
            line_numbers_ = pretty.pretty_list(i.line_number for i in problems),
            are_ = "are" if len(problems) > 1 else "is",
            lines_ = pretty.plural_if("Line", problems),
            lines = problems,
            file_name = file_name,
            dscore = -len(problems),
            type = "interact/indentation/basic_indent_level"
        ))

    # The score is just the sum of the dscores for each message plus the max
    # score (since dscores are negative this makes sense).
    score = max_score + sum(i.dscore for i in result.messages)
    if not allow_negative:
        score = max(0, score)
    result.score = score

    return result

def check_compiles(files, flags = [], ignore_cache = False):
    # Try to compile the program
    compiler_output, executable_path = execute.compile_program(
        files, flags = flags, ignore_cache = ignore_cache
    )

    command = pretty.craft_shell_command(
        execute.create_compile_command(files, flags)
    )

    result = core.TestResult(
        brief = "This test ensures that your code compiles without errors. "
                "Your program was compiled with %s." % (command, ),
        default_message = "**Great job!** Your code compiled cleanly without "
                          "any problems.",
        max_score = 10,
        bulleted_messages = False
    )

    # If the compilation failed
    if executable_path is None:
        if compiler_output:
            compiler_output = pretty.truncate_string(compiler_output)

            result.add_message(
                "Your code did not compile. The compiler outputted the following "
                "errors:\n\n```\n{compiler_output}\n```\n",
                compiler_output = compiler_output,
                dscore = -10
            )
        else:
            result.add_message(
                "Your code did not compile but the compiler did not output "
                "any errors or warnings."
            )

        result.score = 0
    else:
        result.score = 10

    return result;



# Copyright (c) 2013 Galah Group LLC
# Copyright (c) 2013 Other contributers as noted in the CONTRIBUTERS file
#
# This file is part of galah-interact-python.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import interact.standardtests as standardtests
import testcore as testcore
import tempfile
import shutil
import os

import unittest
class TestCheckFilesExist(unittest.TestCase):
    def test_missing_files(self):
        temp_dir = tempfile.mkdtemp()

        try:
            test_files = [
                os.path.join(temp_dir, i) for i in ["hello.txt", "goodbye.txt"]
            ]

            test_result = standardtests.check_files_exist(*test_files)

            self.assertFalse(test_result.is_passing())

            self.assertEquals(len(test_result.messages), 1)
            self.assertItemsEqual(
                test_result.messages[0].kwargs["missing_files"], test_files
            )
        finally:
            shutil.rmtree(temp_dir)

    def test_extant_files(self):
        temp_dir = tempfile.mkdtemp()

        try:
            test_files = [
                os.path.join(temp_dir, i) for i in ["hello.txt", "goodbye.txt"]
            ]

            for i in test_files:
                with open(i, "w") as f:
                    print >> f, "Hello world!"

            test_result = standardtests.check_files_exist(*test_files)

            self.assertTrue(test_result.is_passing())

            self.assertListEqual(test_result.messages, [])
        finally:
            shutil.rmtree(temp_dir)



# Copyright (c) 2013 Galah Group LLC
# Copyright (c) 2013 Other contributers as noted in the CONTRIBUTERS file
#
# This file is part of galah-interact-python.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import tempfile
import subprocess
import shutil
import os
import os.path
import interact.core
import atexit

# Create and set up cleanup code for the cache. The cache stores as keys a
# sorted (alphabetically) tuple of the absolute file paths used to create the
# executable whose absolute path is stored in the value. The directory,
# as returned by os.path.dirname will be deleted once the program exits.
_cache = {}
def _cleanup():
    for i in _cache.values():
        shutil.rmtree(os.path.dirname(i))
atexit.register(_cleanup)

def create_compile_command(files, flags):
    """
    From a list of files and flags, crafts a list suitable to pass into
    subprocess.Popen to compile those files.

    """

    return ["g++"] + flags + ["-o", "main"] + files

def compile_program(files, flags = [], ignore_cache = False):
    """
    Compiles the provided code files. If ignore_cache is False and the program
    has already been compiled with this function, it will not be compiled
    again.

    Returns a two-tuple with the compiler output first and an absolute path to
    the executable second. If the executable was loaded from the cache, the
    compiler output will be None. If the program did not compile, the path will
    be None.

    Note that this function blocks for as long it takes to compile the files
    (unless of course the results are loaded from the cache).

    """

    # If we've already compiled these files don't do it again
    file_tuple = tuple(sorted(files))
    if not ignore_cache and file_tuple in _cache:
        return (None, _cache[file_tuple])

    temp_dir = tempfile.mkdtemp()

    try:
        # We want to always override the name of the output file otherwise we
        # won't know what it's named (though we could try to detect it if it
        # becomes a desirable features.)
        command = create_compile_command(files, flags)

        compiler_job = subprocess.Popen(
            command,
            cwd = temp_dir,
            stdout = subprocess.PIPE,
            stderr = subprocess.STDOUT
        )

        compiler_job.wait()
        if compiler_job.returncode != 0:
            return (compiler_job.stdout.read(), None)

        executable_path = os.path.join(temp_dir, "main")
        _cache[file_tuple] = executable_path

        return (compiler_job.stdout.read(), executable_path)
    except:
        shutil.rmtree(temp_dir)
        raise

def default_run_func(executable, temp_dir):
    """
    Used by the run_program function to create a Popen object that is
    responsible for running the exectuable. temp_dir will be an absolute path to
    a temporary directory that can be used as the current working directory. It
    will be deleted automatically at the end of the run_program function. The
    executable will not be in the directory.

    This function may be overriden to override the default run_func value used
    in the run_program function.

    """

    return subprocess.Popen(
        [executable],
        cwd = temp_dir,
        stdout = subprocess.PIPE,
        stdin = subprocess.PIPE
    )

def run_program(files = None, given_input = "", run_func = None,
        executable = None):
    """
    Executes the given program (if files was specified, the program will be
    compiled first via the compile_program function) and returns a three-tuple
    with standard output first, standard error output second, and the return
    code third (ie: (stdout, stderr, returncode)).

    """

    if (files is None and executable is None) or \
            (files is not None and executable is not None):
        raise TypeError(
            "Either files or executable must be specified, but not both nor "
            "neither."
        )

    # Doing this each time the function runs rather than putting the default
    # in the function header allows users to override default_run_func.
    if run_func is None:
        run_func = default_run_func

    # Compile the given files if we weren't given an executable.
    if executable is None:
        compile_output, executable = compile_program(files)
        if not executable:
            raise RuntimeError("Program did not compile.")

    temp_dir = tempfile.mkdtemp()

    try:
        user_program = run_func(executable, temp_dir)

        stdout, stderr = user_program.communicate(given_input)

        return (stdout, stderr, user_program.returncode)
    finally:
        shutil.rmtree(temp_dir)



#!/usr/bin/env python

import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "galah-interact",
    version = read("VERSION"),
    author = "Galah Group LLC and other contributers",
    author_email = "jsull003@ucr.edu",
    description = (
        "A Python framework for creating test harnesses to grade students' "
        "code."
    ),
    license = "Apache v2.0",
    keywords = "education framework",
    url = "https://www.github.com/galah-group/galah-interact-python",
    packages = ["interact", "interact_test"],
    long_description = read("README.md"),
    classifiers = [
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
    ],
)



# Copyright (c) 2013 Galah Group LLC
# Copyright (c) 2013 Other contributers as noted in the CONTRIBUTERS file
#
# This file is part of galah-interact-python.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Module useful for displaying nice, grammatically correct output.

"""

def pretty_list(the_list, conjunction = "and", none_string = "nothing"):
    """
    Returns a grammatically correct string representing the given list. For
    example...

    >>> pretty_list(["John", "Bill", "Stacy"])
    "John, Bill, and Stacy"
    >>> pretty_list(["Bill", "Jorgan"], "or")
    "Bill or Jorgan"
    >>> pretty_list([], none_string = "nobody")
    "nobody"

    """

    the_list = list(the_list)

    if len(the_list) == 0:
        return none_string
    elif len(the_list) == 1:
        return str(the_list[0])
    elif len(the_list) == 2:
        return str(the_list[0]) + " " + conjunction + " " + str(the_list[1])
    else:
        # Add every item except the last two together seperated by commas
        result = ", ".join([str(s) for s in the_list[:-2]]) + ", "

        # Add the last two items, joined together by a command and the given
        # conjunction
        result += "%s, %s %s" % \
            (str(the_list[-2]), conjunction, str(the_list[-1]))

        return result

def plural_if(zstring, zcondition):
    """
    Returns zstring pluralized (adds an 's' to the end) if zcondition is True or
    if zcondition is not equal to 1.

    Example usage could be ``plural_if("cow", len(cow_list))``.

    """

    # If they gave us a boolean value, just use that, otherwise, assume the
    # value is some integral type.
    if type(zcondition) is bool:
        plural = zcondition
    else:
        plural = zcondition != 1

    return zstring + ("s" if plural else "")

def limit_string_length(string, max_lines = 20, max_characters = 20 * 72):
    # If the string is empty, we don't need to do anything.
    # if not string:
        # return string

    # Negative values are bad
    if max_lines < 0:
        raise TypeError("max_lines cannot be negative, got %d." % (max_lines, ))
    elif max_characters < 0:
        raise TypeError(
            "max_characters cannot be negative, got %d." % (max_lines, )
        )

    nlines_truncated = 0
    lines = string.splitlines()
    if len(lines) > max_lines:
        nlines_truncated = len(lines) - max_lines
        lines = lines[:max_lines]

    ncharacters_truncated = 0
    if len(string) > max_characters:
        ncharacters_truncated = len(string) - max_characters
        string = string[:max_characters]

    return (string, nlines_truncated, ncharacters_truncated)

def truncate_string(string, max_lines = 20, max_characters = 20 * 72):
    new_string, nlines_truncated, ncharacters_truncated = limit_string_length(
        string, max_lines, max_characters
    )

    if ncharacters_truncated == 0 and nlines_truncated == 0:
        return new_string
    else:
        return new_string + "\n---Remaining text truncated---"

def craft_shell_command(command):
    """
    Returns a shell command from a list of arguments suitable to be passed into
    subprocess.Popen. The returned string should only be used for display
    purposes and is not secure enough to actually be sent into a shell.

    """

    return " ".join([escape_shell_string(i) for i in command])

def escape_shell_string(str):
    """
    Escapes a shell string such that it is suitable to be displayed to the user.
    This function **should not** be used to actually feed arguments into a shell
    as this function **is not** secure enough.

    """

    escaped = str.replace("\\", "\\\\").replace("\"", "\\\"")
    if " " in escaped:
        escaped = "\"" + escaped + "\""

    return escaped



# Copyright (c) 2013 Galah Group LLC
# Copyright (c) 2013 Other contributers as noted in the CONTRIBUTERS file
#
# This file is part of galah-interact-python.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Galah Interact is a framework for creating Test Harnesses that grade students'
programming assignments. For general project information, check out the project
page on GitHub: https://github.com/galah-group/galah-interact-python

Galah Interact was originally created by Galah Group LLC and is licensed under
the Apache license version 2.0. A copy of the license is available in the root
directory of the git repo in the file LICENSE, and online at
http://www.apache.org/licenses/LICENSE-2.0. Please ensure your use of this
library is within your rights per that license.

Other contributers have given their valuable time to this project in order to
make it better, and they are listed in the CONTRIBUTERS file in the root
directory of the git repo.

"""

from core import *

import pretty
import execute
import parse
import standardtests
